%ACCEL	Calcula las aceleraciones articulares resultantes de aplicar unos determinados
%     	pares a las articulaciones del manipulador
%
%	QDD = ACCEL(DYN, Q, QD, TORQUE, GRAV) devuelve un vector columna con
%	las aceleraciones articulares que resultan de aplicar al manipulador los pares
%	contenidos en el vector TORQUE.
%
%	Utiliza el mtodo 1 de Walker y Orin para calcular la dinmica del manipulador. Este
%	mtodo es til para la simulacin de la dinmica del manipulador, en conjuncin con
%	una funcin numrica de integracin.
%
%	Con el vector columna GRAV se especifica la aceleracion de la gravedad que sufre el
%	manipulador (por defecto se toma 9.81 m/s2 en direccin -Z, es decir GRAV=[0 0 -9.81]).
%
%	Ver tambin RNE, DYN, ODE45.

%  Copyright (C)	1999 Peter Corke
%						2001 Ivn Maza

function qdd = accel(dyn, q, qd, torque, grav)
	q = q(:)';
	qd = qd(:)';
	M = inertia(dyn, q);	% Clculo de la matriz de masas
   tau = rne(dyn, q, qd, zeros(size(q)), grav);	% 	Clculo de los trminos centrfugos y de Coriolis, 
																%	gravitatorios y de friccin.
	qdd = inv(M) * (torque(:) - tau');
