% FKINE	Calcula la cinemtica directa de un robot manipulador
%
%	FKINE(DH, Q)  calcula la cinemtica directa de una manipulador para un
%	estado dado por el vector de variables articulares Q. Dicho manipulador
%	quedar descrito por su matriz de parmetros de Denavit-Hartenberg (DH).
%
%	Es posible utilizar esta funcin de dos modos:
%
%	a)	Si Q es un vector, entonces es interpretado como el vector de variables 
%		articulares para el que se pretende calcular el modelo directo, y FKINE
%		devuelve la transformacin homognea T correspondiente al ltimo enlace
%		del manipulador. En este caso Q deber ser un vector fila.
% 
%	b)	Si Q es una matriz, cada fila ser interpretada como un vector de varia-
%		bles articulares, y T ser una matriz tridimensional 4x4xm, siendo m el
%		nmero de filas de Q. T es lo que se denominar una trayectoria de trans-
%		formaciones homogneas.
%		Para extraer la transformacin homognea (Ti) que hay "condensada" en la
%		fila i-sima de la matriz T, bastar con escribir:
%
%                            TI = T(:,:,i)
%
%	Empleando la funcin FKINE segn este ltimo modo, es posible obtener todas
%	las transformaciones homogneas correspondientes a una trayectoria dada en 
%	el espacio de variables articulares.
%
%	Ver tambin DH, LINKTRANS.

%	Copyright (C)	Peter Corke 1999
%						Ivn Maza 1999                      

function t = fkine(dh, q)
   
   n = numrows(dh);
	if numcols(q) ~= n,
		error('datos errneos')
	end
	
	if numrows(q) == 1, % si q es solamente un vector fila
		t = eye(4,4);
		for i=1:n,
         t = t * linktrans(dh(i,:), q(i));
		end
	else
		k=1;
		for qv=q',		% para cada punto de la trayectoria dada en variables articulares
			tt = eye(4,4);
			for i=1:n,
            tt = tt * linktrans(dh(i,:), qv(i));
         end
         t(:,:,k) = tt;
         k=k+1;
		end
	end
