%	POL5	Calcula las trayectorias articulares resultantes de emplear polinomios qunticos
%
%	[QCOEF,TIME,Q_T,QD_T,QDD_T] = POL5 (Q,QD,QDD,T0,TIJ,STEP)
%
%	Los parmetros de la funcin son:
%
%	- Q:	matriz con tantas filas como articulaciones, que contiene en cada columna las
%		posiciones articulares deseadas para el punto inicial,los puntos de paso y el
%		punto final.
%	- QD:	matriz con tantas filas como articulaciones, que contiene en cada columna las
%		velocidades articulares deseadas para el punto inicial,los puntos de paso y el
%		punto final.
%	- QDD:	matriz con tantas filas como articulaciones, que contiene en cada columna las
%		aceleraciones articulares deseadas para el punto inicial,los puntos de paso y el
%		punto final.
%	- T0:	parmetro que contiene el instante inicial
%	- TIJ:	Vector que contiene la duracin de los intervalos entre puntos.
%	- STEP:	Paso con el que se calculan la posicin, velocidad y aceleracin resultantes.
%
%	Esta funcin devuelve:
%
%	- QCOEF	matriz de tres dimensiones, que contiene en cada fila los coeficientes del
%		polinomio quntico solucin para cada una de las variables articulares. La primera
%		columna se corresponde con la potencia cbica y la ltima con el trmino indepen-
%		diente. El tercer ndice de la matriz hace referencia a la articulacin a la que
%		le corresponden los coeficientes.
%	- TIME	vector tiempo con los instantes para los que se calculan las matrices Q_T,
%		QD_T y QDD_T que contienen los valores de la posicin, velocidad y aceleracin
%		articular resultantes.
%	- Q_T	matriz que contiene en cada fila los valores de posicin correspondientes a
%		cada articulacin.
%	- QD_T	matriz que contiene en cada fila los valores de velocidad correspondientes a
%		cada articulacin.
%	- QDD_T	matriz que contiene en cada fila los valores de aceleracin correspondientes a
%		cada articulacin.

%	Copyright (C) Ivn Maza 2001

function [Qcoef, time, q_t, qd_t, qdd_t] = pol5 (q, qd, qdd, t0, tij, step)

l = length(tij);
[h,n] = size(q);
[o,m] = size(qd);
[y,z] = size(qdd);
if (n~=m) | (n~=z) | (h~=o) | (h~=y)
   error('Las dimensiones de las matrices de especificacin de posicin, velocidad y aceleracin deben ser idnticas');
end

if (l~=n-1)
   error('La dimensin del vector de duracin de intervalos no es correcta')
end

tfinal = sum (tij)+t0;
time = t0:step:tfinal;


Qcoef = [];


for w=1:h
	t_ini = t0;   
   qaux = q(w,:);
   qdaux = qd(w,:);
   qddaux = qdd(w,:);
   qt = [];
   qdt = [];
   qddt = [];
	for i=1:1:n-1
   	[Qcoef(i,:,w), taux, q_taux, qd_taux, qdd_taux] = pol5aux([qaux(i) qaux(i+1)],[qdaux(i) qdaux(i+1)],[qddaux(i) qddaux(i+1)],[t_ini t_ini+tij(i)],step);
   	qt = [qt q_taux(1:length(q_taux)-1)];
		qdt = [qdt qd_taux(1:length(qd_taux)-1)];
   	qddt = [qddt qdd_taux(1:length(qdd_taux)-1)];         
      t_ini = t_ini+tij(i);
	end
	q_t(w,:) = [qt q_taux(length(q_taux))];
	qd_t(w,:) = [qdt qd_taux(length(qd_taux))];
   qdd_t(w,:) = [qddt qdd_taux(length(qdd_taux))];
end
